import os
import logging
from dotenv import load_dotenv
from pyalby.utils import make_get_request

# Load environment variables from the `.env` file
load_dotenv()

class Account:
    """
    A class to interact with the Alby API to retrieve account information.
    """

    def __init__(self):
        """
        Initializes the Account instance with base URL and access token from environment variables.
        """
        self.base_url = os.getenv("BASE_URL")
        if not self.base_url:
            logging.error("BASE_URL is not set in environment variables.")
            raise ValueError("BASE_URL is required")

        self.access_token = os.getenv('ALBY_ACCESS_TOKEN')
        if not self.access_token:
            logging.error("ALBY_ACCESS_TOKEN is not set in environment variables.")
            raise ValueError("ALBY_ACCESS_TOKEN is required")

        self.headers = {
            "Authorization": f"Bearer {self.access_token}",
            "Content-Type": "application/json",
        }

    def get_value4value(self):
        """
        Retrieves the user's Lightning Address and keysend information.

        Returns:
            dict: A dictionary containing keysend_pubkey, keysend_custom_key,
                  keysend_custom_value, and lightning_address.
        """
        url = f"{self.base_url}/user/value4value"
        return make_get_request(url, self.headers)

    def get_balance(self):
        """
        Retrieves the balance of the user's wallet.

        Returns:
            dict: A dictionary containing balance, currency, and unit.
        """
        url = f"{self.base_url}/balance"
        return make_get_request(url, self.headers)

    def get_account_summary(self):
        """
        Retrieves the account summary including balance, transaction count, and boostagram count.

        Returns:
            dict: A dictionary containing balance, boostagrams_count, currency,
                  invoices_count, last_invoice_at, transactions_count, and unit.
        """
        url = f"{self.base_url}/user/summary"
        return make_get_request(url, self.headers)

    def get_personal_info(self):
        """
        Retrieves the user's personal information including e-mail address, display name, avatar,
        nostr pubkey, and value4value information.

        Returns:
            dict: A dictionary containing identifier, email, name, avatar, keysend_custom_key,
                  keysend_custom_value, keysend_pubkey, lightning_address, and nostr_pubkey.
        """
        url = f"{self.base_url}/user/me"
        return make_get_request(url, self.headers)



import os
import requests
import logging
from dotenv import load_dotenv
from pyalby.utils import make_post_request

# Load environment variables
load_dotenv()

class Payment:
    """
       A class to interact with the Alby API to do payments.
       """
    def __init__(self):
        """
        Initializes the Payment instance with base URL and access token from environment variables.
        """
        self.base_url = os.getenv("BASE_URL")
        if not self.base_url:
            logging.error("BASE_URL is not set in environment variables.")
            raise ValueError("BASE_URL is required")

        self.access_token = os.getenv('ALBY_ACCESS_TOKEN')
        if not self.access_token:
            logging.error("ALBY_ACCESS_TOKEN is not set in environment variables.")
            raise ValueError("ALBY_ACCESS_TOKEN is required")

        self.headers = {
            "Authorization": f"Bearer {self.access_token}",
            "Content-Type": "application/json",
        }

    def bolt11_payment(self, invoice):
        """Pay a lightning invoice (bolt11).

        Args:
            invoice (str): The BOLT11 invoice string to be paid.

        Returns:
            dict: Information about the successfully paid invoice.
        """
        url = f"{self.base_url}/payments/bolt11"
        data = {"invoice": invoice}
        return make_post_request(url, data, self.headers)

    def keysend_payment(self, amount, destination, memo=None, custom_records=None):
        """Send a spontaneous payment (keysend).

        Args:
            amount (int): Amount in satoshis.
            destination (str): Destination pubkey (starts with 02 or 03).
            memo (str, optional): Internal memo for reference.
            custom_records (dict, optional): Custom key-value records.

        Returns:
            dict: Information about the successfully completed keysend payment.
        """
        url = f"{self.base_url}/payments/keysend"
        data = {
            "amount": amount,
            "destination": destination
        }
        if memo:
            data["memo"] = memo
        if custom_records:
            data["custom_records"] = custom_records

        return make_post_request(url, data, self.headers)

    def multi_keysend_payment(self, keysends):
        """Make multiple keysend payments in a single request.

        Args:
            keysends (list of dict): A list of keysend payment objects.

        Returns:
            list of dict: Each object represents an individual payment response.
        """
        url = f"{self.base_url}/payments/keysend/multi"
        data = {"keysends": keysends}

        response = requests.post(url, json=data, headers=self.headers)
        response.raise_for_status()
        return response.json()



import requests
import logging

def make_get_request(url, headers):
    logging.info(f"Calling GET API URL: {url} with headers: {headers}")
    try:
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        logging.debug(f"Response Status Code: {response.status_code}")
        logging.debug(f"Response Headers: {response.headers}")
        logging.debug(f"Response Body: {response.text}")
        return response.json()
    except requests.exceptions.HTTPError as http_err:
        logging.error(f"HTTP error occurred: {http_err} - Response: {response.text}")
    except requests.exceptions.ConnectionError as conn_err:
        logging.error(f"Connection error occurred: {conn_err}")
    except requests.exceptions.Timeout as timeout_err:
        logging.error(f"Timeout error occurred: {timeout_err}")
    except requests.exceptions.RequestException as req_err:
        logging.error(f"An error occurred: {req_err}")
    except Exception as err:
        logging.error(f"An unexpected error occurred: {err}")

    return err


def make_post_request(url, data, headers):
    logging.info(f"Calling POST API URL: {url} with headers: {headers} and data: {data}")
    try:
        # Remove any optional arguments with None values
        data = {k: v for k, v in data.items() if v is not None}
        response = requests.post(url, json=data, headers=headers)
        response.raise_for_status()
        logging.debug(f"Response Status Code: {response.status_code}")
        logging.debug(f"Response Headers: {response.headers}")
        logging.debug(f"Response Body: {response.text}")
        return response.json()
    except requests.exceptions.HTTPError as http_err:
        logging.error(f"HTTP error occurred: {http_err} - Response: {response.text}")
    except requests.exceptions.ConnectionError as conn_err:
        logging.error(f"Connection error occurred: {conn_err}")
    except requests.exceptions.Timeout as timeout_err:
        logging.error(f"Timeout error occurred: {timeout_err}")
    except requests.exceptions.RequestException as req_err:
        logging.error(f"An error occurred: {req_err}")
    except Exception as err:
        logging.error(f"An unexpected error occurred: {err}")

    return err



import logging
import os

# Setting up logging for the package
logger = logging.getLogger(__name__)
handler = logging.StreamHandler()
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

# Allow setting logging level from an environment variable or default to INFO
log_level = os.getenv('LOG_LEVEL', 'INFO')
logger.setLevel(log_level)

# Making classes easily importable
from .account import Account
from .invoice import Invoice
from .payment import Payment

# List of all available classes for `from pyalby import *`
__all__ = ['Account', 'Invoice', 'Payment']


import os
import logging
from dotenv import load_dotenv

from pyalby.utils import make_get_request, make_post_request

# Load environment variables from the `.env` file
load_dotenv()


class Invoice:
    """
       A class to interact with the Alby API to create and get inovoices.
       """
    def __init__(self):
        """
        Initializes the Invoice instance with base URL and access token from environment variables.
        """
        self.base_url = os.getenv("BASE_URL")
        if not self.base_url:
            logging.error("BASE_URL is not set in environment variables.")
            raise ValueError("BASE_URL is required")

        self.access_token = os.getenv('ALBY_ACCESS_TOKEN')
        if not self.access_token:
            logging.error("ALBY_ACCESS_TOKEN is not set in environment variables.")
            raise ValueError("ALBY_ACCESS_TOKEN is required")

        self.headers = {
            "Authorization": f"Bearer {self.access_token}",
            "Content-Type": "application/json",
        }

    def create_invoice(
            self,
            amount,  # Required
            description,  # Required
            currency="btc",  # Optional
            memo=None,  # Optional
            comment=None,  # Optional
            payer_name=None,  # Optional
            payer_email=None,  # Optional
            payer_pubkey=None,  # Optional
    ):
        """Create a new invoice for receiving lightning payments.

        Args:
            amount (int): Required amount in satoshis.
            description (str): Required description text (included in the BOLT11 invoice).
            currency (str, optional): Currency of the invoice. Defaults to "btc".
            memo (str, optional): Same as "description". Defaults to None.
            comment (str, optional): Arbitrary text to save alongside the invoice. Defaults to None.
            payer_name (str, optional): Name of the payer. Defaults to None.
            payer_email (str, optional): Email of the payer. Defaults to None.
            payer_pubkey (str, optional): Nostr or node pubkey of the payer. Defaults to None.

        Returns:
            dict: JSON response containing the invoice data.
        """
        url = f"{self.base_url}/invoices"
        data = {
            "amount": amount,
            "description": description,
            "currency": currency,
            "memo": memo,
            "comment": comment,
            "payer_name": payer_name,
            "payer_email": payer_email,
            "payer_pubkey": payer_pubkey
        }
        return make_post_request(url, data, self.headers)

    def get_incoming_invoices(self):
        """Get the list of incoming invoices with optional filters.

        Returns:
            list: List of incoming invoices.
        """
        url = f"{self.base_url}/invoices/incoming"
        return make_get_request(url, self.headers)


    def get_outgoing_invoices(self):
        """Get the list of outgoing invoices with optional filters.

        Returns:
            list: List of outgoing invoices.
        """
        url = f"{self.base_url}/invoices/outgoing"
        return make_get_request(url, self.headers)

    def get_specific_invoice(self, payment_hash):
        """Get details about a specific invoice.

        Args:
            payment_hash (str): Payment hash of the specific invoice.

        Returns:
            dict: Details of the specified invoice.
        """
        url = f"{self.base_url}/invoices/{payment_hash}"
        return make_get_request(url, self.headers)

    def decode_invoice(self, bolt11_invoice):
        """Decode a Bolt11 invoice.

        Args:
            bolt11_invoice (str): Bolt11 invoice string to decode.

        Returns:
            dict: Decoded invoice information.
        """
        url = f"{self.base_url}/decode/bolt11/{bolt11_invoice}"
        return make_get_request(url, self.headers)


